@@ -5,16 +5,16 @@ function(x)
   paste('"', x, '"', sep = "")
 
 setGeneric("toJSON",
-function(x, container = length(x) > 1  || length(names(x)) > 0, collapse = "\n", ...)
+function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L)
   standardGeneric("toJSON"))
 
 setMethod("toJSON", "NULL",
-           function(x, container = length(x) > 1  || length(names(x)) > 0, collapse = "\n", ...) {
-             "null"
+           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+             if(container) "[ null ] " else "null"
            })
 
 setMethod("toJSON", "ANY",
-           function(x, container = length(x) > 1  || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
 
              if(isS4(x)) {
                paste("{", paste(dQuote(slotNames(x)), sapply(slotNames(x), function(id) toJSON(slot(x, id), ...)), sep = ": "),
@@ -27,7 +27,7 @@ setMethod("toJSON", "ANY",
 
 
 setMethod("toJSON", "integer",
-           function(x, container = length(x) > 1 || length(names(x)) > 1, collapse = "\n  ", ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 1, collapse = "\n  ", ..., .level = 1L) {
 
              if(container) {
                 if(length(names(x)))
@@ -42,7 +42,7 @@ setMethod("toJSON", "integer",
 setOldClass("hexmode")
 
 setMethod("toJSON", "hexmode",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n   ", ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n   ", ..., .level = 1L) {
              tmp = paste("0x", format(x), sep = "")
              if(container) {
                 if(length(names(x)))
@@ -55,12 +55,12 @@ setMethod("toJSON", "hexmode",
 
 
 setMethod("toJSON", "factor",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
-             toJSON(as.character(x), container, ...)
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+             toJSON(as.character(x), container, collapse, ..., .level = .level)
            })
 
 setMethod("toJSON", "logical",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
              tmp = ifelse(x, "true", "false")
              if(container) {
                 if(length(names(x)))
@@ -72,7 +72,7 @@ setMethod("toJSON", "logical",
            })
 
 setMethod("toJSON", "numeric",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L) {
              tmp = formatC(x, digits = digits)
              if(container) {
                 if(length(names(x)))
@@ -85,9 +85,10 @@ setMethod("toJSON", "numeric",
 
 
 setMethod("toJSON", "character",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L) {
              tmp = gsub("\\\n", "\\\\n", x)
-             tmp = gsub('"', '\\\\"', tmp)             
+             tmp = gsub('"', '\\\\"', tmp)
+             tmp = gsub('(\\\\)', '\\1\\1', tmp)                          
              tmp = dQuote(tmp)
              if(container) {
                 if(length(names(x)))
@@ -102,7 +103,7 @@ setMethod("toJSON", "character",
 
 # Symbols.
 setMethod("toJSON", "name",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
              as.character(x)
            })
 
@@ -126,14 +127,14 @@ setMethod("toJSON", "matrix",
            })
 
 setMethod("toJSON", "list",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
                 # Degenerate case.
              if(length(x) == 0) {
                           # x = structure(list(), names = character()) gives {}
                 return(if(is.null(names(x))) "[]" else "{}")
              }
              
-             els = sapply(x, toJSON, ...)
+             els = sapply(x, toJSON, ..., .level = .level + 1L)
 
              if(all(sapply(els, is.name)))
                names(els) = NULL
